!cpu 6502
!to "fastldr",plain
*=$801

		secldr2   = 7		;7th sector is fastest when loaded from bootsector but can be any of them
		adrldr2   = 9		;page of second stage, must be 9 if loaded from bootsector

		adrlo     = $26		;constant from boot prom
		adrhi     = $27		;constant from boot prom
		slot      = $2b		;constant from boot prom
		tmpsec    = $3c		;constant from boot prom
		partial2  = $f9		;copy of partial1
		partial1  = $fa		;sectors to read from one track
		total     = $fb		;total sectors to read
		step      = $fc		;state for stepper motor
		tmptrk    = $fd		;temporary copy of current track
		curtrk    = $fe		;current track (must initialise before calling seek)
		reqtrk    = $ff		;requested track*2 for seek
		addrtbl   = $100	;requires 16 bytes
		nibtbl    = $2d6	;constant from boot prom
		seekpatch = $901	;offset of first instruction in second page

		maxsec    = $10		;sectors per track, usually 10 but 0b for Olympic Decathlon, 0c for Karateka

		newtrk    = $42		;track for main stage
		newnum    = $1f		;number of sectors in main stage
		newsec    = 1		;first sector in main stage
		newadr    = $61		;load address for main stage
		newstart  = $6100	;start address for main stage


		;this section is specific to bootsectors
		;load second stage sector to $900, enabling track seek
		ldx #$60		;rts (slot 6 bootsectors can omit this instruction)
		stx seekpatch		;disable seek until it's loaded
		tax			;1 sector
		ldy #secldr2
		lda #adrldr2
		jsr seekread
		jmp ldr2
		;

seekread	stx total
calcpart	sty partial1
		pha
		tya
		pha
		clc
		adc total
		cmp #maxsec + 1
		bcc +
		lda #maxsec
		sbc partial1
		tax
+		stx partial1
		stx partial2
		jsr seek
		ldx #$0f
		lda #0
-		sta addrtbl, x
		dex
		bpl -
		pla
		tay
		pla
-		ldx sectbl, y
		sta addrtbl, x
		tax
		inx
		iny
		txa
		dec partial1
		bne -
		pha

read		ldx slot
read1		clc
read2		php
-		lda $c08c, x
		bpl -
--		eor #$d5
		bne -
-		lda $c08c, x
		bpl -
		cmp #$aa
		bne --
		nop
-		lda $c08c, x
		bpl -
		cmp #$96
		bne check_mode
		ldy #3
readhdr		lda $c08c, x
		bpl readhdr
		rol
		sta tmpsec
-		lda $c08c, x
		bpl -
		and tmpsec
		dey
		bne readhdr
		plp
		tay
		lda addrtbl, y
		beq read1
		sta adrhi
		lda #0
		sta addrtbl, y
		sec
		bcs read2
check_mode	plp
		bcc read1
		eor #$ad
link_br		bne read
		ldy #$56
--		sty tmpsec
-		ldy $c08c, x
		bpl -
		eor nibtbl, y
		ldy tmpsec
		dey
		sta nibtbl + 42, y
		bne --
--		sty tmpsec
-		ldy $c08c, x
		bpl -
		eor nibtbl, y
		ldy tmpsec
		sta (adrlo), y
		iny
		bne --
--		ldx #$56
-		dex
		bmi --
		lda (adrlo), y
		lsr nibtbl + 42, x
		rol
		lsr nibtbl + 42, x
		rol
		sta (adrlo), y
		iny
		bne -
		dec total
		dec partial2
		bne link_br
		pla
		inc reqtrk
		inc reqtrk
		ldx total
		beq seekret
		jmp calcpart

delay		ldx #$13
-		dex
		bne -
		sec
		sbc #1
		bne delay
seekret		rts

sectbl		!byte 0, $0d, $0b, 9, 7, 5, 3, 1, $0e, $0c, $0a, 8, 6, 4, 2, $0f

seek		lda curtrk
		cmp reqtrk
		beq seekret
		lda #0
seekpatch	sta step
copy_cur	lda curtrk
		sta tmptrk
		sec
		sbc reqtrk
		beq seek_done
		bcs +
		eor #$ff
		inc curtrk
		bcc ++
+		adc #$fe
		dec curtrk
++		cmp step
		bcc +
		lda step
+		cmp #$0c
		bcs +
		tay
		sec
+		jsr step_cur
		lda step1, y
		jsr delay
		lda tmptrk
		clc
		jsr step_tmp
		lda step2, y
		jsr delay
		inc step
		bne copy_cur
seek_done	jsr delay
		clc
step_cur	lda curtrk
step_tmp	and #3
		rol
		ora slot
		tax
		lda $c080, x
		rts

step1		!byte 1, $30, $28, $24, $20, $1e, $1d, $1c, $1c, $1c, $1c, $1c
step2		!byte $70, $2c, $26, $22, $1f, $1e, $1d, $1c, $1c, $1c, $1c, $1c

ldr2		jmp ldr2
		;replace with something like the following:
		;sty curtrk
		;lda #newtrk
		;sta reqtrk
		;ldx #newnum
		;ldy #newsec
		;lda #newadr
		;jsr seekread
		;jmp newstart
